# Generated by Django 5.1.4 on 2025-01-02 02:22

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MarkovModel',
            fields=[
                ('hash', models.CharField(max_length=255, primary_key=True, serialize=False)),
                ('text', models.TextField()),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]



from django.db import models


class MarkovModel(models.Model):
    hash = models.CharField(max_length=255, primary_key=True)
    text = models.TextField()
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.hash



AGENTS = [
  "AI2Bot",
  "Ai2Bot-Dolma",
  "Amazonbot",
  "Applebot",
  "Applebot-Extended",
  "Bytespider",
  "CCBot",
  "ChatGPT-User",
  "Claude-Web",
  "ClaudeBot",
  "Diffbot",
  "DuckAssistBot",
  "FacebookBot",
  "FriendlyCrawler",
  "GPTBot",
  "Google-Extended",
  "GoogleOther",
  "GoogleOther-Image",
  "GoogleOther-Video",
  "ICC-Crawler",
  "ISSCyberRiskCrawler",
  "ImagesiftBot",
  "Kangaroo Bot",
  "Meta-ExternalAgent",
  "Meta-ExternalFetcher",
  "OAI-SearchBot",
  "PanguBot",
  "PerplexityBot",
  "PetalBot",
  "Scrapy",
  "SemrushBot",
  "Sidetrade indexer bot",
  "Timpibot",
  "VelenPublicWebCrawler",
  "Webzio-Extended",
  "YouBot",
  "anthropic-ai",
  "cohere-ai",
  "cohere-training-data-crawler",
  "iaskspider/2.0",
  "img2dataset",
  "omgili",
  "omgilibot"
]



from django import template
import logging
from django_llm_poison.markov import poisoned_string
from django_llm_poison.agents import AGENTS

register = template.Library()
logger = logging.getLogger(__name__)


def request_from_bot(request) -> bool:
    if request.GET.get("poison"):
        return True
    user_agent = request.META.get("HTTP_USER_AGENT", "")
    for agent in AGENTS:
        if agent.lower() in user_agent.lower():
            logger.info("Serving poisoned content to %s", user_agent)
            return True

    return False


def do_poison(parser, token):
    nodelist = parser.parse(("endpoison",))
    parser.delete_first_token()
    return PoisonNode(nodelist)


class PoisonNode(template.Node):
    def __init__(self, nodelist):
        self.nodelist = nodelist

    def render(self, context):
        output = self.nodelist.render(context)
        if context.get("request") and request_from_bot(context["request"]):
            return poisoned_string(output)
        else:
            return output


register.tag("poison", do_poison)



from django.core.cache import cache
import re
import markovify
import hashlib
from django.utils.html import strip_tags

from markovify.splitters import split_into_sentences
from django_llm_poison.models import MarkovModel


def generate_combined_model():
    combined_model = None
    for model in MarkovModel.objects.all():
        if combined_model is None:
            combined_model = markovify.Text.from_json(model.text)
        else:
            combined_model = markovify.combine(
                [combined_model, markovify.Text.from_json(model.text)]
            )
    return combined_model.compile(inplace=True)


def get_combined_model():
    if not (model := cache.get("combined_model")):
        model = generate_combined_model()
        cache.set("combined_model", model)
    return model


def build_model_for_content(value: str):
    hash = hashlib.sha1(value.encode()).hexdigest()
    stripped = strip_tags(value)
    if not MarkovModel.objects.filter(hash=hash).exists():
        model = markovify.Text(stripped)
        markov_model = MarkovModel.objects.create(hash=hash, text=model.to_json())
        cache.delete("combined_model")
        return markov_model


def poisoned_string(value: str) -> str:
    build_model_for_content(value)
    model = get_combined_model()
    if model is None:
        return value

    sentences = []
    tags = re.split(r"(<[^>]+>)", value)
    for t in tags:
        sentences.extend(split_into_sentences(t))
    for idx, sentence in enumerate(sentences):
        if len(sentence) > 5 and idx != 0 and idx % 3 == 0:
            try:
                generated = model.make_sentence(
                    init_state=tuple(sentence.split()[: model.chain.state_size]),
                    test_output=False,
                )
                sentences[idx] = generated
            except KeyError:
                pass

    return " ".join(sentences)



import pytest
from django.template.loader import render_to_string

from django_llm_poison.models import MarkovModel
from django_llm_poison.markov import build_model_for_content
from django_llm_poison.templatetags.poison import request_from_bot


@pytest.fixture
def markov_model():
    content = render_to_string("index.html", {})
    return build_model_for_content(content)


@pytest.mark.django_db
def test_models_created(client):
    assert MarkovModel.objects.count() == 0
    response = client.get("/?poison=1")
    assert response.status_code == 200
    assert MarkovModel.objects.count() == 1

    response = client.get("/short/?poison=1")
    assert response.status_code == 200
    assert MarkovModel.objects.count() == 2


@pytest.mark.django_db
def test_posioned_content_served(client, markov_model):
    # Test non-bots get the same content
    response = client.get("/")
    response2 = client.get("/")
    assert response.content == response2.content

    # Test bots get different content
    bot_response = client.get("/?poison=1")
    assert bot_response.content != response.content


def test_user_agent_belongs_to_bot(rf):
    request = rf.get("/", HTTP_USER_AGENT="GPTBot")
    assert request_from_bot(request)

    request = rf.get("/", HTTP_USER_AGENT="Firefox")
    assert not request_from_bot(request)


def test_poison_paramter_acts_like_bot(rf):
    request = rf.get("/?poison=1", HTTP_USER_AGENT="Firefox")
    assert request_from_bot(request)
